package org.bss.esb;

import org.bss.esb.xml.BeanType;
import org.bss.esb.xml.BeanType.Property;
import org.bss.esb.xml.EsbConfigurationDocument;
import org.bss.esb.xml.EsbConfigurationDocument.EsbConfiguration;
import org.bss.esb.xml.EsbConfigurationDocument.EsbConfiguration.Beans;
import org.bss.esb.xml.EsbConfigurationDocument.EsbConfiguration.RetryPolicies;
import org.bss.esb.xml.RetryPolicyType;
import org.bss.esb.xml.RouteType;
import org.bss.esb.xml.RouteType.TransportHandler;

import java.lang.reflect.Method;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

/**
 * Helper class for initializing the ESB configuration by creating
 * appropriate bean classes as specified in the EsbConfiguration XML.
 * It then puts all the created beans in a {@link java.util.Map}.
 * Information about the configured routes and retry policies is also
 * placed in separate {@link java.util.Map} objects.
 *
 * @author Balwinder Sodhi
 * @version 1.0
 * @see RouteInfo
 * @see RetryPolicyInfo
 */
public class EsbConfigManager {

    /**
     * Map containing the {@link RouteInfo} objects that are created as
     * per the configuration in the <code>EsbConfiguration</code> XML.
     */
    private HashMap routeMap = new HashMap();

    /**
     * Map holding the bean instances that are configured in the
     * <code>EsbConfiguration</code> XML.
     */
    private HashMap beansMap = new HashMap();

    /**
     * Map holding the {@link RetryPolicyInfo} instances as
     * per the configuration in the <code>EsbConfiguration</code> XML.
     */
    private HashMap retryPolicyMap = new HashMap();

    /**
     * Returns the map holding {@link RouteInfo} objects.
     *
     * @return
     */
    public final Map getRouteMap() {
        return routeMap;
    }

    /**
     * Returns the map holding the bean instances as per the configuration
     * in EsbConfiguration XML.
     *
     * @return
     */
    public final Map getBeansMap() {
        return this.beansMap;
    }

    /**
     * Returns the map holding the {@link RetryPolicyInfo} objects as per
     * the configuration in EsbConfiguration XML.
     *
     * @return
     */
    public final Map getRetryPolicyMap() {
        return retryPolicyMap;
    }

    /**
     * Clears all the object maps maintained by this class.
     */
    public void clear() {

        if (routeMap != null) {
            routeMap.clear();
        }
        if (beansMap != null) {
            beansMap.clear();
        }
        if (retryPolicyMap != null) {
            retryPolicyMap.clear();
        }
    }

    /**
     * Initializes the various object maps as per the configuration defined
     * in the EsbConfiguration XML.
     *
     * @param configUrl
     * @throws Exception
     */
    public void init(String configUrl) throws Exception {

        EsbConfigurationDocument doc =
                EsbConfigurationDocument.Factory.parse(new URL(configUrl));
        initBeans(doc);

    }

    /**
     * Initializes the bean classes.
     *
     * @param doc The EsbConfigurationDocument XmlBean parsed from
     *            the EsbConfiguration XML file.
     * @throws Exception
     */
    private void initBeans(EsbConfigurationDocument doc) throws Exception {

        EsbConfiguration config = doc.getEsbConfiguration();
        Beans beans = config.getBeans();
        if (beans != null) {

            BeanType[] beanArray = beans.getBeanArray();
            for (int i = 0; i < beanArray.length; i++) {

                beansMap.put(beanArray[i].getName(),
                        createBean(beanArray[i]));

            }
        }

        initRoutes(config.getRouteArray());
        RetryPolicies policies = config.getRetryPolicies();
        if (policies != null) {
            initRetryPolicies(policies.getRetryPolicyArray());
        }

    }

    /**
     * Initializes the {@link RouteInfo} map
     *
     * @param routes
     */
    private void initRoutes(RouteType[] routes) {

        RouteInfo rinfo = null;
        for (int i = 0; i < routes.length; i++) {
            
            rinfo = new RouteInfo();
            if (routes[i].getAuthConstraint() != null) {
                rinfo.setAuthorizedPrincipals(routes[i].
                        getAuthConstraint().getPrincipals().split(","));
            }
            if (routes[i].getDeadLetterDestination() != null) {
                rinfo.setDeadLetterDestination(routes[i].
                        getDeadLetterDestination().getRouteName());
            }
            rinfo.setRetryPolicyName(routes[i].getRetryPolicyRef());
            rinfo.setRouteName(routes[i].getName());
            if (routes[i].getTransformerRef() != null) {
                rinfo.setTransformer(routes[i].getTransformerRef());
            }
            TransportHandler[] tRefs =
                    routes[i].getTransportHandlerArray();
            String[] tInfos =
                    new String[tRefs.length];
            for (int j = 0; j < tRefs.length; j++) {

                tInfos[j] = tRefs[j].getBeanName();

            }
            rinfo.setTransportHandlerBeanNames(tInfos);

            this.routeMap.put(rinfo.getRouteName(), rinfo);
        }

    }

    /**
     * Initializes the {@link RetryPolicyInfo} map.
     *
     * @param policies
     */
    private void initRetryPolicies(RetryPolicyType[] policies) {

        for (int i = 0; i < policies.length; i++) {

            retryPolicyMap.put(policies[i].getName(),
                    new RetryPolicyInfo(policies[i].getName(),
                            policies[i].getMaxRetries(),
                            policies[i].getRetryInterval()));

        }
    }

    /**
     * Creates a bean class as per the configuration in the
     * EsbConfiguration XML.
     *
     * @param bean
     * @return
     * @throws Exception
     */
    private Object createBean(BeanType bean) throws Exception {

        Object o = Class.forName(bean.getClassName()).newInstance();
        Property[] props = bean.getPropertyArray();
        for (int i = 0; i < props.length; i++) {
            String type = props[i].getType().trim();
            if ("float".equalsIgnoreCase(type) ||
                    "java.lang.Float".equalsIgnoreCase(type)) {

                invokeMethod(o, "set" + props[i].getName(),
                        new Class[]{Float.class}, new Object[]{
                            new Float(props[i].getValue().trim())});

            } else if ("int".equalsIgnoreCase(type) ||
                    "java.lang.Integer".equalsIgnoreCase(type)) {

                invokeMethod(o, "set" + props[i].getName(),
                        new Class[]{Integer.class}, new Object[]{
                            new Integer(props[i].getValue().trim())});

            } else if ("long".equalsIgnoreCase(type) ||
                    "java.lang.Long".equalsIgnoreCase(type)) {

                invokeMethod(o, "set" + props[i].getName(),
                        new Class[]{Float.class}, new Object[]{
                            new Float(props[i].getValue().trim())});

            } else if ("double".equalsIgnoreCase(type) ||
                    "java.lang.Double".equalsIgnoreCase(type)) {

                invokeMethod(o, "set" + props[i].getName(),
                        new Class[]{Double.class}, new Object[]{
                            new Double(props[i].getValue().trim())});

            } else if ("short".equalsIgnoreCase(type) ||
                    "java.lang.Short".equalsIgnoreCase(type)) {

                invokeMethod(o, "set" + props[i].getName(),
                        new Class[]{Short.class}, new Object[]{
                            new Short(props[i].getValue().trim())});

            } else if ("char".equalsIgnoreCase(type) ||
                    "java.lang.Character".equalsIgnoreCase(type)) {

                invokeMethod(o, "set" + props[i].getName(),
                        new Class[]{Character.class}, new Object[]{
                            new Character(props[i].getValue().toCharArray()[0])});

            } else if ("java.lang.String".equalsIgnoreCase(type)) {

                invokeMethod(o, "set" + props[i].getName(),
                        new Class[]{String.class}, new Object[]{
                            props[i].getValue().trim()});

            } else {

                throw new IllegalArgumentException("Invalid pproperty type: " +
                        type + ". Should be only primitive or wrapper of it.");

            }
        }
        return o;
    }

    /**
     * Uses reflection to invoke a given method on a given class.
     *
     * @param obj        Instance of the class on which to call the method
     * @param methodName Name of the method to be invoked
     * @param argTypes   Class[] containing the argument types
     * @param args       Object[] containing the actual param values
     * @throws Exception
     */
    public static Object invokeMethod(Object obj, String methodName,
                                      Class[] argTypes, Object[] args) throws Exception {

        Method meth = obj.getClass().getMethod(methodName, argTypes);
        Object retVal = meth.invoke(obj, args);
        return retVal;

    }
} 
