package org.bss.esb;


import org.bss.esb.management.EsbRouterMonitorMBean;
import org.bss.esb.management.MBeanFactory;
import org.bss.esb.transform.TransformException;
import org.bss.esb.transform.TransformHandler;
import org.bss.esb.transport.EndPointNotFoundException;
import org.bss.esb.transport.TransportException;
import org.bss.esb.transport.TransportHandler;
import org.bss.esb.util.JmsUtil;
import org.bss.esb.xml.EnvConfigDocument;

import javax.ejb.CreateException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.JMSException;
import javax.management.MBeanException;
import javax.naming.InitialContext;
import java.net.URL;
import java.util.Date;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This MDB is the starting point in the ESB flow. It listens for the messages
 * on the ESB input queue and routes them to the appropriate destination.
 * The inbound message is expected to supply some "key" as a message
 * property. This key is used to lookup a route from the routing table. The
 * message is then sent to that route. In case of any failures, the message
 * is sent to either the redelivery queue, deadletter queue or error queue
 * depending upon the type of failure. The startup configuration for ESB is
 * specified in an XML file whose URL has to be given in the environment
 * entry named "EnvConfigUrl" in this MDB's deployment descriptor.
 *
 * @author Balwinder Sodhi
 * @version 1.0
 * @see EsbConfigManager
 * @see TransportHandler
 */
public class EsbRouter implements MessageDrivenBean, MessageListener {

    /**
     * EsbConfigManager instance.
     */
    private EsbConfigManager esbConfigManager;

    /**
     * EsbRouterMonitorMBean instance.
     */
    private EsbRouterMonitorMBean mbean;

    /**
     * Name of the input message property that will hold the routing key
     */
    private String ROUTING_KEY;

    /**
     * Name of the input message's long property where we'll put the
     * redelivery time for it.
     */
    private String REDELIVERY_TIME_PROP;

    /**
     * Name of the input message's long property where we'll put the
     * redelivery attempts count.
     */
    private String REDELIVERY_TRIES_PROP;

    private String AUTH_PRINCIPAL_PROP;

    /**
     * Routing key for the error route.
     */
    private String errorRouteName;

    /**
     * Routing key for the redelivery route.
     */
    private String redeliveryRouteName;

    /**
     * Routing key for the redelivery request route.
     */
    private String redeliveryRequestRouteName;

    /**
     * Routing key for the deadletter route.
     */
    private String deadLetterRouteName;

    /**
     * URL of the EsbConfiguration XML file.
     */
    private String esbConfigUrl;

    /**
     * URL of the EnvConfig XML file.
     */
    private String envConfigUrl;

    /**
     * Logger instance.
     */
    private Logger LOG = Logger.getLogger(getClass().getName());

    /**
     * JNDI initial naming context.
     */
    private InitialContext ic;

    /**
     * Timestamp of the first message received by this MDB instance since
     * the last call to {@link #resetTimings()}.
     */
    private long timeLow = -1;

    /**
     * Timestamp of the latest message received by this MDB instance since
     * the last call to {@link #resetTimings()}.
     */
    private long timeHigh = -1;

    /**
     * Total number of messages received by this MDB instance since
     * the last call to {@link #resetTimings()}.
     */
    private int count = 0;

    protected MessageDrivenContext ctx;

    public void setMessageDrivenContext(MessageDrivenContext ctx) {

        this.ctx = ctx;

    }

    /**
     * This method performs the intialization tasks for this MDB:
     * <pre>
     * 1) The config.xml is read and parsed into an XmlBean in this method.
     * <p/>
     * 2) From this XmlBean, the rest of the confiuration parameters are read.
     * <p/>
     * 3) The EsbConfiguration XML is read and used to initialize the
     * EsbConfigManager instance.
     * <p/>
     * 4) The EsbRouterMonitorMBean is created and registered.
     * <p/>
     * </pre>
     *
     * @throws CreateException
     */
    public void ejbCreate() throws CreateException {

        try {

            LOG.fine("Creating the router MDB.");
            ic = new InitialContext();

            envConfigUrl = (String) ic.
                    lookup("java:comp/env/EnvConfigUrl");
            URL configUrl = new URL(envConfigUrl);
            EnvConfigDocument config =
                    EnvConfigDocument.Factory.parse(configUrl);
            redeliveryRouteName =
                    config.getEnvConfig().getRedeliveryRouteName();
            deadLetterRouteName =
                    config.getEnvConfig().getDeadLetterRouteName();
            errorRouteName = config.getEnvConfig().getErrorRouteName();
            redeliveryRequestRouteName =
                    config.getEnvConfig().getRedeliveryRequestRouteName();
            ROUTING_KEY = config.getEnvConfig().getMessagePeoprties().
                    getDestination();
            REDELIVERY_TRIES_PROP = config.getEnvConfig().
                    getMessagePeoprties().getRedeliveryAttempts();
            REDELIVERY_TIME_PROP = config.getEnvConfig().
                    getMessagePeoprties().getRedeliveryTime();
            AUTH_PRINCIPAL_PROP = config.getEnvConfig().
                    getMessagePeoprties().getAuthPrincipal();

            esbConfigUrl = config.getEnvConfig().getEsbConfigurationUrl();
            esbConfigManager = new EsbConfigManager();
            esbConfigManager.init(esbConfigUrl);
            mbean = MBeanFactory.getEsbRouterMonitorMBean(this, ic);

        } catch (Exception xex) {

            CreateException c = new CreateException();
            c.initCause(xex);
            throw c;

        }
    }

    /**
     * Unregisters the EsbRouterMonitorMBean instance.
     */
    public void ejbRemove() {

        try {

            mbean.unregister();
            LOG.fine("Removed the MDB.");

        } catch (Exception ex) {
            ex.printStackTrace();
        }

    }

    /**
     * Processes the messages received by the ESB on its configured input
     * queue. Based on the routing key spepcified in the message, it tries
     * to send the message to appropriate route. Any irrecoverable failue
     * causes the transaction to be rolled back.
     *
     * @param msg The input message received on the ESB's input queue.
     */
    public void onMessage(Message msg) {

        String routeName = null;
        try {

            count++;
            long t1 = msg.getJMSTimestamp();
            routeName = msg.getStringProperty(ROUTING_KEY);
            LOG.fine("Received the message for destination: " + routeName);
            if (timeLow == -1) {

                timeLow = t1;
                timeHigh = t1;

            } else if (t1 < timeLow) {

                timeLow = t1;

            }

            if (t1 > timeHigh) {
                timeHigh = t1;
            }

            routeMessage(msg, routeName);

        } catch (Throwable ex) {

            LOG.log(Level.FINEST, ex.getLocalizedMessage());
            ctx.setRollbackOnly();
        }
    }

    /**
     * Routes the <code>msg</code> based on the supplied route name. Folowing
     * steps are taken:
     * <pre>
     * 1) Lookup the transport handler bean names for the given route from the
     * maps maintained in the helper class EsbConfigManager.
     * <p/>
     * 2) Invoke the transportMessage() of each TransportHandler bean passing it
     * the message we received.
     * <p/>
     * 3) If EsbConfigManager doesn't have any entry for the given route
     * then send the message to configured deadletter route.
     * <p/>
     * 4) In case a TransportHandler failes, then queue the message for retry.
     * </pre>
     *
     * @param msg       The message to be routed.
     * @param routeName Name of the route where this message is to be sent.
     * @throws EndPointNotFoundException
     * @throws TransportException
     */
    private void routeMessage(Object msg, String routeName)
            throws EndPointNotFoundException, TransportException {

        RouteInfo route = null;
        try {

            route = (RouteInfo) esbConfigManager.getRouteMap().get(routeName);
            if (isAuthorized(route, msg)) {

                if (route.getTransformer() != null) {

                    msg = applyTransformation(msg, route.getTransformer());

                }
                dispatchMessage(msg, route);

            } else {
                LOG.warning("Unauthorized message request received: " + routeName);
            }

        } catch (EndPointNotFoundException epnfe) {

            try {

                routeMessage(msg, deadLetterRouteName);

            } catch (EndPointNotFoundException epne) {

                LOG.log(Level.FINEST, "Couldn't send the message to " +
                        "dead-letter queue.", epne);

                routeMessage(msg, errorRouteName);

            } catch (TransportException te) {

                LOG.log(Level.FINEST, "Couldn't send the message to " +
                        "dead-letter queue.", te);
                routeMessage(msg, errorRouteName);

            }


        } catch (Throwable ex) {
            LOG.log(Level.FINEST, "Couldn't send message to route: " + routeName
                    + ".", ex);
            queueMessageForRetry(msg, route);
        }
    }

    /**
     * Dispatches the received message to a given route by using the transport
     * handlers configured for it in the EsbConfiguration XML.
     *
     * @param msg
     * @param route
     * @throws EndPointNotFoundException
     * @throws TransportException
     */
    private void dispatchMessage(Object msg, RouteInfo route)
            throws EndPointNotFoundException, TransportException {

        String[] handlers = route.getTransportHandlerBeanNames();
        if (handlers != null) {
            for (int i = 0; i < handlers.length; i++) {

                try {

                    TransportHandler h = (TransportHandler)
                            esbConfigManager.getBeansMap().get(handlers[i]);
                    h.transportMessage(msg);

                } catch (TransportException te) {

                    LOG.log(Level.FINEST, "", te);
                    queueMessageForRetry(msg, route);

                }

            }
        } else {
            throw new IllegalStateException("No transport handler found.");
        }
    }

    /**
     * Applies the transformation to the received message as configured
     * in the EsbConfiguration XML.
     *
     * @param msg         Received message
     * @param transformer Name of the transformer bean
     * @return Message after it has been processed by the transformer bean.
     * @throws TransformException
     */
    private Object applyTransformation(Object msg, String transformer)
            throws TransformException {

        TransformHandler tf = (TransformHandler) esbConfigManager.getBeansMap().
                get(transformer);
        if (tf != null) {
            return tf.transformMessage(msg);
        } else {
            throw new IllegalStateException("Transformer bean not found: " +
                    transformer);
        }

    }

    /**
     * Does the authorization check for the given route. In this method we
     * do a very simplistic check. However, it is possible to do more
     * elaborate checks via custom code that you can invoke from here.
     *
     * @param route
     * @param msg
     * @return
     * @throws JMSException
     */
    private boolean isAuthorized(RouteInfo route, final Object msg) throws JMSException {

        boolean auth = false;
        String[] principals = route.getAuthorizedPrincipals();
        LOG.fine("Principals: " + Arrays.asList(principals));
        if (principals != null && principals.length > 0) {

            Message message = (Message) msg;
            /**
             * Get some message property which represents the sender principal
             * and the verify if that matches with any of the elements in
             * principals array.
             */
            for (int i = 0; i < principals.length; i++) {
                if (principals[i].equals(message.getStringProperty(AUTH_PRINCIPAL_PROP))) {
                    auth = true;
                    break;
                }
            }
        }
        return auth;
    }

    /**
     * Queues the <code>msg</code> for retry. Following is the logic for
     * queuing a message M, destined for a given route R, for redelivery:
     * <pre>
     * <p/>
     * 1) A property holding the past redelivery attempts
     * count for message M is read from it.
     * <p/>
     * 2) Lookup the retry policy for the given route.
     * <p/>
     * 3) IF theres no property holding the past redelivery attempts count
     * for message M AND the maximum retries is zero as per retry policy
     * THEN we send the message M to the dead-letter route and subsequent
     * steps are omitted.
     * <p/>
     * 4) ELSE IF the redelivery attempts count for M exceeds the maximum
     * retries allowed as per the retry policy THEN we send the M to
     * dead-letter route associated with route R.
     * <p/>
     * 5) ELSE :
     *      i.	The redelivery attempts count is incremented by one
     * <p/>
     *      ii.	Calculate the next redelivery time based on the retry
     *      interval in the retry policy.
     * <p/>
     *      iii.	Both the incremented attempts count and the next
     *      redelivery time are set as properties in M.
     * <p/>
     *      iv.	Send the message M to the redelivery route. For this we
     *      lookup the appropriate TransportHandler as usual from the
     *      EsbConfigManager for the redelivery route.
     * <p/>
     *      v.	Send a redelivery request message to the redelivery request
     *      route. This message contains just the future time (calculated in
     *      step ii above as java.lang.Long) when the message M should be
     *      scheduled for redelivery.
     * <p/>
     * 6) If theres any error at any point in the redelivery attempt, then
     * we send the message M to dead letter route associated with the route R.
     * If the send to dead letter route also fails due to some reason, then M
     * is sent to the error route. If the send to error route too fails then
     * the transaction of the onMessage() of EsbRouter MDB is rolled back
     * which results in the message M being put back on the ESBs input queue.
     * <p/>
     * </pre>
     *
     * @param message Message to be queued for redelivery.
     * @param route   The route for which this message was destined.
     * @throws EndPointNotFoundException
     * @throws TransportException
     */
    private void queueMessageForRetry(Object message, RouteInfo route)
            throws EndPointNotFoundException, TransportException {

        Message msg = (Message) message;
        String dlEndPoint = route.getDeadLetterDestination();
        try {

            int c = 0;
            if (msg.propertyExists(REDELIVERY_TRIES_PROP)) {

                c = msg.getIntProperty(REDELIVERY_TRIES_PROP);

            }

            RetryPolicyInfo policy = (RetryPolicyInfo) esbConfigManager.
                    getRetryPolicyMap().get(route.getRetryPolicyName());
            if (c < policy.getMaxRetries()) {

                JmsUtil.makeMessagePropertiesWritable(msg);
                msg.setIntProperty(REDELIVERY_TRIES_PROP, c + 1);
                long reDeliverAt = System.currentTimeMillis() +
                        policy.getRetryInterval() * 1000;
                msg.setLongProperty(REDELIVERY_TIME_PROP, reDeliverAt);
                LOG.fine("Redelivery of message [" + msg.getJMSMessageID() +
                        "] will be attempted at: " +
                        new Date(msg.getLongProperty(REDELIVERY_TIME_PROP)));
                dispatchMessage(new Long(reDeliverAt), (RouteInfo)
                        esbConfigManager.getRouteMap().get(redeliveryRequestRouteName));
                dispatchMessage(msg, (RouteInfo)
                        esbConfigManager.getRouteMap().get(redeliveryRouteName));

            } else {
                routeMessage(msg, dlEndPoint);
            }

        } catch (Exception je) {


            LOG.log(Level.SEVERE, "Problem queueing the message for re-delivery.",
                    je);
            routeMessage(msg, dlEndPoint);
        }
    }

    /******************** MBean Methods ************************/

    /**
     * Returns the total number of messages received by this MDB
     * instance since the last call to {@link #resetTimings()}.
     *
     * @return message count
     */
    public int getMessageCount() {

        LOG.fine("Count: " + count);
        return count;
    }

    /**
     * Returns the timestamp of the first message received by this MDB
     * instance since the last call to {@link #resetTimings()}.
     */
    public long getLowestTime() {

        LOG.fine("Lowest Time: " + timeLow);
        return timeLow;
    }

    /**
     * Returns the timestamp of the latest message received by this MDB
     * instance since the last call to {@link #resetTimings()}.
     */
    public long getHighestTime() {

        LOG.fine("Highest Time: " + timeHigh);
        return timeHigh;
    }

    /**
     * Resets the values of <code>count</code>, <code>timeHigh</code>
     * and <code>timeLow</code> members.
     */
    public void resetTimings() {

        timeHigh = -1;
        timeLow = -1;
        count = 0;
        LOG.fine("Timings reset.");
    }

    /**
     * Updates the EsbConfigurationDocument instance of this MDB by reloading
     * the EsbRoutingTable.xml file from the configured location. It then
     * recreates the {@link EsbConfigManager} instance of this MDB to
     * refltect the configuration changes found in the reloaded routing
     * table file.
     *
     * @throws MBeanException
     */
    public void updateRoutingTable() throws MBeanException {

        LOG.fine("Updating the routing table.");
        try {

            EsbConfigManager configManager = new EsbConfigManager();
            configManager.init(esbConfigUrl);

            if (esbConfigManager != null) {
                esbConfigManager.clear();
            }
            esbConfigManager = configManager;

        } catch (Exception ie) {
            throw new MBeanException(ie);
        }

    }

    /**
     * Returns the URL string of the routing table xml file location.
     *
     * @return URL string for the routing table xml file location.
     */
    public String getRoutingTableLocation() {
        return esbConfigUrl;
    }

    /**
     * Sets the location URL from where the routing table xml file
     * will be loaded.
     *
     * @param urlString URL string for the routing table xml file location.
     */
    public void setRoutingTableLocation(String urlString) {
        esbConfigUrl = urlString;
    }
}


