package org.bss.esb.management;

import org.bss.esb.EsbRouter;
import java.util.logging.Logger;
import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import weblogic.management.MBeanHome;

/**
 * This is the implementation class for the EsbRouterMonitorMBean.
 * 
 * @version 1.0
 * @author Balwinder Sodhi
 */
public class EsbRouterMonitor implements EsbRouterMonitorMBean { 
    
    /** MBeanHome instance used for accessing this MBean from the server. */
    private MBeanHome mbeanHome;
    
    /** {@link EsbRouter} instance associated with this MBean. */
    private EsbRouter router;
    
    /** Logger instance used by this class. */
    private Logger LOG = Logger.getAnonymousLogger();
    
    /** Type of this MBean. */
    public static final String MBEAN_TYPE = "EsbRouterMonitor";
    
    /**
     * Constructor for this class. It looks up the MBeanHome from the
     * server and registers this MBean instance with the server.
     * 
     * @param router The EsbRouter instance that will be managed by
     * this MBean.
     * @param ic The JNDI initial naming context.
     * @exception NamingException
     */
    public EsbRouterMonitor(EsbRouter router, InitialContext ic) 
            throws NamingException{
        
        this.router = router;
        mbeanHome = (MBeanHome) ic.lookup(MBeanHome.ADMIN_JNDI_NAME);
        registerMBean();
        
    }
    
    /**
     * @see {@link EsbRouter#getHighestTime()}
     */
    public long getHighestTime() {
        return router.getHighestTime();
    }
    

    /**
     * @see {@link EsbRouter#getLowestTime()}
     */
    public long getLowestTime() {
        return router.getLowestTime();
    }

    /**
     * @see {@link EsbRouter#getMessageCount()}
     */
    public int getMessageCount() {
        return router.getMessageCount();
    }

    /**
     * @see {@link EsbRouter#getHighestTime()}
     */
    public void resetTimings() {
        router.resetTimings();
    }
    
    /**
     * @see {@link EsbRouter#updateRoutingTable()}
     */
    public void updateRoutingTable() throws MBeanException {
        router.updateRoutingTable();
    }
    
    /**
     * Registers this MBean instance with the server.
     */
    public void registerMBean() {

        try {

            MBeanServer server = mbeanHome.getMBeanServer();
            ObjectName objName =
                new ObjectName(mbeanHome.getDomainName() +":Name="+
                    MBEAN_TYPE+"-"+this.hashCode()+",Type="+MBEAN_TYPE);
            
            // Register MBean so MBean can be accessed by remote managers
            try {
                server.registerMBean(this, objName);
                LOG.info("Registered: "+objName.getCanonicalName());
            } catch (InstanceAlreadyExistsException ignore) {
                ignore.printStackTrace();
            }

        } catch (Exception xex) {
            xex.printStackTrace();
        }
        
    }
    
    /**
     * Unregisters this MBean instance from the server.
     */
    public void unregister() {

        try {
            
            MBeanServer server = mbeanHome.getMBeanServer();
            ObjectName objName =
                new ObjectName(mbeanHome.getDomainName() +":Name="+
                    MBEAN_TYPE+"-"+this.hashCode()+",Type="+MBEAN_TYPE);
            
            try {
                server.unregisterMBean(objName);
                LOG.info("Un-registered: "+objName.getCanonicalName());
            } catch (InstanceNotFoundException ignore) {
                ignore.printStackTrace();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        
    }
    
    /**
     * @see {@link EsbRouter#getRoutingTableLocation()}
     */
    public String getRoutingTableLocation() {
        
        return router.getRoutingTableLocation();
        
    }

    /**
     * @see {@link EsbRouter#setRoutingTableLocation(String)}
     */
    public void setRoutingTableLocation(String path) {
        
        router.setRoutingTableLocation(path);
        
    }
} 
