package org.bss.esb.management;

import org.bss.esb.redelivery.RedeliveryScheduler;
import weblogic.management.MBeanHome;

import javax.management.*;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.logging.Logger;

/**
 * This is the implementation class for the RedeliverySchedulerMonitorMBean.
 * It is used to manage a {@link RedeliveryScheduler} instance.
 * 
 * @version 1.0
 * @author Balwinder Sodhi
 * @see RedeliveryScheduler
 */
public class RedeliverySchedulerMonitor 
        implements RedeliverySchedulerMonitorMBean { 
    
    /** {@link RedeliveryScheduler} instance that's managed by this MBean. */
    private RedeliveryScheduler scheduler;
    
    /** Logger instance used by this class. */
    private Logger LOG = Logger.getAnonymousLogger();
    
    /** Type of this MBean */
    public static final String MBEAN_TYPE = "RedeliverySchedulerMonitor";
    
    /**
     * Constructor that initializes the RedeliveryScheduler member.
     * 
     * @param scheduler The RedeliveryScheduler instance to be managed.
     */
    public RedeliverySchedulerMonitor(RedeliveryScheduler scheduler) {
        
        this.scheduler = scheduler;                
        
    }
    
    /**
     * ***************************************************
     *                  MBean methods
     * ***************************************************
     */
    
    /**
     * Stops the RedeliveryScheduler instance.
     * @see {@link RedeliveryScheduler#stopRedeliveryScheduler()}
     */
    public void stopRedeliveryScheduler() {
        
        scheduler.stopRedeliveryScheduler();
        
    }
    
    /**
     * Resets the RedeliveryScheduler instance.
     * @see {@link RedeliveryScheduler#stopRedeliveryScheduler()}
     */
    public void resetRedeliveryScheduler() {
        
        scheduler.resetRedeliveryScheduler();
        
    }    
    
    /**
     * Registers this MBean instance with the server.
     * @param ic The JNDI initial naming context.
     * @exception NamingException
     * @exception MalformedObjectNameException
     * @exception NotCompliantMBeanException
     * @exception MBeanRegistrationException
     */
    public void register(InitialContext ic) throws NamingException, 
            MalformedObjectNameException, NotCompliantMBeanException, 
                    MBeanRegistrationException {
        
        MBeanHome mbeanHome = 
                (MBeanHome) ic.lookup(MBeanHome.ADMIN_JNDI_NAME);
        MBeanServer server = mbeanHome.getMBeanServer();
        ObjectName objName =
            new ObjectName(mbeanHome.getDomainName() +
                ":Name="+MBEAN_TYPE+"-"+this.hashCode()+
                        ",Type="+MBEAN_TYPE);
        
        // Register MBean so MBean can be accessed by remote managers
        try {
            server.registerMBean(this, objName);
            LOG.info("RedeliverySchedulerMonitor registered as MBean: "+
                    objName.getCanonicalName());
            
        } catch (InstanceAlreadyExistsException ignore) {
            ignore.printStackTrace();
        }
        
    }

    /**
     * Unregisters this MBean instance from the server.
     * @param ic The JNDI initial naming context.
     */
    public void unregister(InitialContext ic) {

        try {
            
            MBeanHome mbeanHome = 
                    (MBeanHome) ic.lookup(MBeanHome.ADMIN_JNDI_NAME);
            MBeanServer server = mbeanHome.getMBeanServer();
            ObjectName objName =
                new ObjectName(mbeanHome.getDomainName() +
                ":Name="+MBEAN_TYPE+"-"+this.hashCode()+
                        ",Type="+MBEAN_TYPE);
            
            try {
                server.unregisterMBean(objName);
                LOG.info("Un-registered: "+objName.getCanonicalName());
            } catch (InstanceNotFoundException ignore) {
                ignore.printStackTrace();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        
    }
    
} 
