package org.bss.esb.redelivery;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;
import javax.naming.InitialContext;
import java.util.logging.Logger;

/**
 * This MDB processes the redelivery requests that are sent by the ESB router
 * over the configured topic. The redelivery request is a JMS ObjectMessage
 * that contains a java.lang.Long as payload. This java.lang.Long object
 * corresponds to:
 * <pre>
 * 1. The time when a message from the configured redelivery queue will be
 * received.
 *
 * 2. The match value of a long property of the message to be redelivered.
 * The name of this property is configurable.
 *
 * </pre>
 *
 * @version 1.0
 * @author Balwinder Sodhi
 * @see RedeliveryScheduler
 * @see RedeliveryTask
 * @see java.util.Timer
 *
 */
public class RedeliveryRequestProcessor implements MessageDrivenBean,
		MessageListener {

    /** Scheduler instance */
    private RedeliveryScheduler scheduler;

    /** Initial JNDI naming context reference */
    private InitialContext ic;

    /** Logger instance to be used for any debug or other log messages. */
    private Logger LOG = Logger.getLogger(getClass().getName());

    protected MessageDrivenContext ctx;

    /**
     * We intialilize the JNDI initial naming context instance and
     * the {@link RedeliveryScheduler} instance in this method.
     *
     * @exception CreateException
     */
    public void ejbCreate() throws CreateException {

        try {

            ic = new InitialContext();
            scheduler = RedeliveryScheduler.getInstance();
            LOG.fine("Created the MDB");

        } catch (Exception ex) {

            CreateException ce = new CreateException();
            ce.initCause(ex);
            throw ce;
        }
    }

    /**
     * The RedeliveryScheduler is stopped on removal of this MDB instance
     * and the MBean associated with that scheduler is also unregistered.
     *
     * @exception EJBException
     */
    public void ejbRemove() throws EJBException {

        try {

            scheduler.stopRedeliveryScheduler();
            scheduler.unregisterMBean(ic);
            LOG.fine("Removed the MDB.");

        } catch (Exception ex) {
            EJBException ee = new EJBException(ex);
            throw ee;
        }
    }

	public void setMessageDrivenContext(MessageDrivenContext ctx) {

		this.ctx = ctx;

	}

    /**
     * Process the redelivery request message that is receoived. It simply
     * calls the <code>scheduleMessageForDelivery</code> method of the
     * RedeliveryScheduler instance that we have.
     *
     * @param msg The ObjectMessage instance containing a java.lang.Long
     */
    public void onMessage(Message msg) {

        try {
            LOG.fine("Received redelivery scheduling request.");
            ObjectMessage oMsg = (ObjectMessage)msg;
            Long deliveryTime = (Long) oMsg.getObject();
            scheduler.scheduleMessageForDelivery(deliveryTime.longValue());

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

}


