package org.bss.esb.redelivery;

import org.bss.esb.management.RedeliverySchedulerMonitor;
import org.bss.esb.xml.EnvConfigDocument;
import java.net.URL;
import java.util.Date;
import java.util.Enumeration;
import java.util.Timer;
import java.util.logging.Logger;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Queue;
import javax.jms.QueueBrowser;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSession;
import javax.naming.InitialContext;

/**
 * This is a Singleton class that performs the scheduling of the
 * {@link RedeliveryTask}s using a single {@link java.util.Timer} instance.
 * At the one time initialization, it browses the ESB's configured
 * redelivery queue to look for any pending messages. Based on the redelivery
 * properties of each message, it schedules a {@link RedeliveryTask} instance.
 * After the startup clients can use this class for scheduling new
 * {@link RedeliveryTask}s.
 * 
 * @version 1.0
 * @author Balwinder Sodhi
 * @see RedeliveryTask
 */
public class RedeliveryScheduler {
    
    /** URL of the EnvConfig.xml file. */
    private String esbConfigUrl;
    
    /** JNDI name of the input queue of the scheduler. */
    private String schedulerInputDestination;
    
    /** JNDI name of the output queue of the scheduler. */
    private String schedulerOutputDestination;
    
    /** JNDI name of the QueueConnectionFactory. */
    private String qcf;
    
    /** Timer instance used by the scheduler. */
    private Timer timer;
    
    /** 
     * Name of the message property that holds the redelivery 
     * time for a message. 
     */
    private String REDELIVERY_TIME_PROP;

    /** 
     * Name of the message property that holds the redelivery 
     * attempts count for a message. 
     */
    private String REDELIVERY_TRIES_PROP;
    
    /** Single RedeliveryScheduler instance that everybody uses. */
    private static RedeliveryScheduler instance;
    
    /** Is this scheduler instance in cancelled state. */
    private boolean cancelled;
    
    /** Instance of the RedeliverySchedulerMonitor MBean. */
    private RedeliverySchedulerMonitor mbean;
    
    /** Logger instance. */
    private Logger LOG = Logger.getLogger(getClass().getName());
    
    /**
     * Private constructor of this class.
     * At the one time initialization, it browses the ESB's configured
     * redelivery queue to look for any pending messages. Based on the redelivery
     * properties of each message, it schedules a {@link RedeliveryTask} instance.
     * After the startup the clients can use this class for scheduling new
     * {@link RedeliveryTask}s.
     * 
     * @exception Exception
     */
    private RedeliveryScheduler() throws Exception {
        
        init();
        LOG.fine("Created the RedeliveryScheduler instance: "+hashCode());
    }
    
    /**
     * Returns the singleton instance of this class.
     * 
     * @return RedeliveryScheduler
     * @exception Exception
     */
    public static RedeliveryScheduler getInstance() throws Exception {
        
        synchronized (RedeliveryScheduler.class) {
            if (instance == null) {
                instance = new RedeliveryScheduler();
            }
        }
        return instance;
    }
    
    /**
     * 
     */
    public void  scheduleMessageForDelivery(long deliveryTime) 
            throws RedeliverySchedulingException {
        
        try {
            
            synchronized(RedeliveryScheduler.class) {
                if (cancelled) {
                    resetRedeliveryScheduler();
                }
            }
            String selector = REDELIVERY_TIME_PROP + " = " +deliveryTime;
            RedeliveryTask task = new RedeliveryTask(qcf, 
                    schedulerInputDestination, schedulerOutputDestination, 
                            selector);
            Date deliverAt = new Date(deliveryTime);
            timer.schedule(task, deliverAt);
            
        } catch (Exception ex) {
            
            ex.printStackTrace();
            RedeliverySchedulingException rse = 
                    new RedeliverySchedulingException(ex);
            throw rse;
            
        }        
        LOG.fine("Scheduled message for delivery at "+
                new Date(deliveryTime));
        
    }
    
    /**
     * This method first reads the EnvConfig.xml from location given in
     * the environment entry <code>java:comp/env/EnvConfigUrl</code>,
     * then it reads the JNDI names of the configured QueueConnectionFactory,
     * ESB's input queue, redelivery queue etc. from the EnvConfig.xml.
     * After that it invokes initializePendingTasks() method to schedule the
     * necessary {@link RedeliveryTask}s for each of the pending messages 
     * existing in the redelivery queue.
     * 
     * @exception Exception
     */
    private void init() throws Exception {

        InitialContext ic = new InitialContext();
        
        esbConfigUrl = (String)ic.
                lookup("java:comp/env/EnvConfigUrl");
        LOG.fine("Initializing using config file from "+esbConfigUrl);
        URL configUrl = new URL(esbConfigUrl);
        EnvConfigDocument config =
                EnvConfigDocument.Factory.parse(configUrl);
        schedulerInputDestination = config.getEnvConfig().
                getRedeliveryQueue();
        schedulerOutputDestination = config.getEnvConfig().
                getEsbInputQueue();
        this.qcf = config.getEnvConfig().getEsbQueueConnectionFactory();
        REDELIVERY_TRIES_PROP = config.getEnvConfig().
                getMessagePeoprties().getRedeliveryAttempts();
        REDELIVERY_TIME_PROP = config.getEnvConfig().
                getMessagePeoprties().getRedeliveryTime();
        timer = new Timer();
        initializePendingTasks(ic);
        mbean = new RedeliverySchedulerMonitor(this);
        mbean.register(ic);
        
    }
    
    /**
     * This method browses the ESB's configured redelivery queue to
     * look for any pending messages. Based on the redelivery properties 
     * of each message, it schedules a {@link RedeliveryTask} instance.
     * After the startup the clients can use this class for scheduling new
     * {@link RedeliveryTask}s.
     * 
     * @param ic The JNDI initial naming context.
     */
    private void initializePendingTasks(InitialContext ic) {
        
        QueueConnection connection = null;
        try {
            LOG.fine("Initializing the redelivery tasks for existing "+
                    "messages in the "+schedulerInputDestination+ " queue.");

            Queue inputQueue = (Queue)ic.lookup(schedulerInputDestination);
            QueueConnectionFactory qcf = (QueueConnectionFactory)ic.lookup(this.qcf);
            
            connection = qcf.createQueueConnection();
            connection.start();
            QueueSession inputSession = connection.createQueueSession(false, 
                    QueueSession.AUTO_ACKNOWLEDGE);                
    
            QueueBrowser browser = inputSession.createBrowser(inputQueue);
            Enumeration messages = browser.getEnumeration();
            Message msg = null;
            while (messages.hasMoreElements()) {
                
                msg = (Message) messages.nextElement();
                
                try {
                    
                    long deliverAt = msg.getLongProperty(REDELIVERY_TIME_PROP);
                    scheduleMessageForDelivery(deliverAt);
                    
                } catch (RedeliverySchedulingException rse) {
                    rse.printStackTrace();
                } catch (JMSException jme) {
                    jme.printStackTrace();
                }
            }
            
        } catch (Exception je) {
            
            je.printStackTrace();
            
        } finally {

            try {
                
                connection.close();
                
            } catch (Exception ignore) {
                
            }
            
        }
    }
    
    /**
     * Unregisters the RedeliverySchedulerMonitorMBean instance 
     * associated with this RedeliveryScheduler.
     * 
     * @param ic The JNDI initial naming context.
     */
    public void unregisterMBean(InitialContext ic) {
        mbean.unregister(ic);
    }
    
    /**
     * Stops the {@link java.util.Timer} instance associated with this
     * RedeliveryScheduler instance.
     */
    public void stopRedeliveryScheduler() {
        timer.cancel();
        cancelled = true;
        LOG.fine("RedeliveryScheduler stopped.");
    }
    
    /**
     * It first cancels the {@link java.util.Timer} instance associated 
     * with this RedeliveryScheduler instance, then assigns a new instance
     * of the {@link java.util.Timer} to the member.
     */
    public void resetRedeliveryScheduler() {
        timer.cancel();
        timer = new Timer(); 
        cancelled = false;
        LOG.fine("RedeliveryScheduler has been reset.");
    }    

}


