package org.bss.esb.redelivery;

import org.bss.esb.util.JmsUtil;
import java.io.IOException;
import java.util.TimerTask;
import java.util.logging.Logger;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueReceiver;
import javax.jms.QueueSession;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * This class represents a task that can be scheduled by {@link java.util.Timer}
 * instance. When scheduled, this task will do the following:
 * <pre>
 * 1) Receive a message from the configured ESB redelivery queue.
 * A selector that is passed while creating this task instance is used for
 * receiving the message.
 * 
 * 2) If a message is received, then that is sent to the ESB's input queue.
 * 
 * </pre>
 * 
 * @version 1.0
 * @author Balwinder Sodhi
 * @see RedeliveryScheduler
 */
public class RedeliveryTask extends TimerTask {
    
    /** QueueConnectionFactory JNDI name. */
    private String qcfName;
    
    /** JNDI name of the configured redelivery queue. */
    private String redeliveryQueueName;
    
    /** JNDI name of the ESB's input queue. */
    private String esbInputQueue;
    
    /** 
     * Selector string for the receiver that will pick message 
     * from ESB input queue. 
     */
    private String selector;
    
    /** Logger instance. */
    private Logger LOG = Logger.getLogger(getClass().getName());
    
    /**
     * Constructor that initializes the members.
     * 
     * @param qcfName JNDI name of the QueueConnectionFactory.
     * @param redeliveryQueueName JNDI name of the redelivery queue.
     * @param esbInputQueue JNDI name of the ESB's input  queue.
     * @param selector Selector string for creating the receiver.
     */
    public RedeliveryTask(String qcfName, String redeliveryQueueName, 
            String esbInputQueue, String selector) throws NamingException,
                    JMSException, IOException {
        
        this.qcfName = qcfName;
        this.redeliveryQueueName = redeliveryQueueName;
        this.esbInputQueue = esbInputQueue;
        this.selector = selector;
    }
    
    /**
     * It does the following in that order:
     * <pre>
     * 1) Creates the JMS QueueConnection with the supplied factory name.
     * 2) Looks up the JMS queue for the supplied name.
     * 3) Creates a JMS QueueReceiver with the supplied selector.
     * 4) Receives (without wait) the message using the above receiver.
     * 5) If a message is received, then the ESB's input Queue is looked up
     * and a JMS QueueSender is created which is used to send the message 
     * that was received in step #4 above.
     * </pre>
     */
    public void run() {

        QueueConnection con = null;
        QueueSession sess = null;
        try {
            
            LOG.info("Selector is ["+selector+"]. Hashcode: "+hashCode()+
                    ". Queue: "+redeliveryQueueName);
            con = JmsUtil.getQueueConnection(qcfName, true);
            sess = con.createQueueSession(true, 
                    QueueSession.AUTO_ACKNOWLEDGE);
            InitialContext ic = new InitialContext();
            Queue redeliveryQueue = (Queue)ic.lookup(redeliveryQueueName);
            
            QueueReceiver receiver = 
                sess.createReceiver(redeliveryQueue, selector);
            Message msg = receiver.receiveNoWait();
            if (msg != null) {

                Queue esbInQueue = (Queue)ic.lookup(esbInputQueue);
                sess.createSender(esbInQueue).send(msg);
                sess.commit();
                LOG.info("Message redelivered to "+esbInputQueue);

            } else {

                LOG.warning("No message was received with selector: "+
                        selector+" on queue "+redeliveryQueueName);
            }
            
        } catch (Exception ex) {
            ex.printStackTrace();
            try {
                if (sess != null)
                    sess.rollback();
                
            } catch (JMSException je) {
                je.printStackTrace();
            }
            
        } finally {
            try {
                con.close();
            } catch (Exception ignore) {
                ignore.printStackTrace();
            }
        }
    }

} 
