package org.bss.esb.util;

import javax.jms.*;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;

/**
 * Utility class for performing common JMS operations. This class provides
 * methods for: 
 * <pre>
 * 1. Creating a JMS connection (Queue/Topic) given a queue connection 
 * factory's JNDI name
 * 
 * 2. Get a JMS message's properties.
 * 
 * 3. Set the entries found in a given Map as JMS message's properties after
 * detecting the type of each value in the supplied Map.
 * 
 * 4. Make a JMS message properties-writable, without losing the original 
 * properties set on the JMS message.
 * 
 * </pre>
 * 
 * @version 1.0
 * @author Balwinder Sodhi
 */
public class JmsUtil { 

    /**
     * Makes the JMS message properties-writable without losing the originally
     * set properties on this JMS message.
     * 
     * @param msg The JMS message to be made properties-writable
     */
    public static void makeMessagePropertiesWritable(Message msg) 
            throws JMSException {
        
        HashMap props = getMessageProperties(msg);
        msg.clearProperties();
        setMessageProperties(props, msg);
        
    }
    
    /**
     * Returns all the properties that are set a given JMS messages, 
     * as HashMap.
     * 
     * @param msg The JMS message whose properties are to be obtained.
     * @return HashMap Contains the message's properties
     * @exception JMSException
     */
    public static HashMap getMessageProperties(Message msg) throws JMSException {
        
        Enumeration names = msg.getPropertyNames();
        HashMap props = new HashMap();        
        String name = null;
        while (names.hasMoreElements()) {
            
            name = names.nextElement().toString();
            props.put(name, msg.getObjectProperty(name));
            
        }
        
        return props;
    }
    
    /**
     * Sets the entries found in <code>props</code> as the properties
     * of <code>msg</code>. Depending on the type of the value found
     * in <code>props</code> entries, appropriate setter is called on the
     * JMS message. For example if the <code>props</code> has the key-value
     * pairs as:
     * <pre>
     * {"MyIntProperty", java.lang.Integer(2234)},
     * {"MyFloatProperty", java.lang.Float(44.99)}
     * 
     * then these will be added to msg as:
     * msg.setIntProperty("MyIntProperty", java.lang.Integer(2234).intValue());
     * msg.setFloatProperty("MyFloatProperty", java.lang.Float(44.99).floatValue());
     * 
     * </pre>
     * 
     * @param props The HashMap containing the properties to be set.
     * @param msg The JMS message on which the properties are to be set.
     * @exception JMSException
     */
    public static void setMessageProperties(HashMap props, Message msg) 
            throws JMSException {
                
        Iterator it = props.keySet().iterator();
        while (it.hasNext()) {

            String name = it.next().toString();
            Object val = props.get(name);
            if (val instanceof Boolean) {
                
                msg.setBooleanProperty(name, ((Boolean)val).booleanValue());
                
            } else if (val instanceof Byte) {
                
                msg.setByteProperty(name, ((Byte)val).byteValue());
                
            } else if (val instanceof Short) {
                
                msg.setShortProperty(name, ((Short)val).shortValue());
                
            } else if (val instanceof Integer) {
                
                msg.setIntProperty(name, ((Integer)val).intValue());
                
            } else if (val instanceof Long) {
                
                msg.setLongProperty(name, ((Long)val).longValue());
                
            } else if (val instanceof Float) {
                
                msg.setFloatProperty(name, ((Float)val).floatValue());
                
            } else if (val instanceof Double) {
                
                msg.setDoubleProperty(name, ((Double)val).doubleValue());
                
            } else if (val instanceof String) {
                
                msg.setStringProperty(name, (String)val);
                
            }

        }
    }
    
    /**
     * Returns a JMS QueueConnection object created after looking up the
     * given queue connection factory.
     * 
     * @param qcfName JNDI name of the queue connection factory that will
     * be used for creating the queue connection.
     * @param started If set to <code>true</code> then the new connection
     * is returned after caling <code>start()</code> on it.
     * @return QueueConnection
     * @exception NamingException
     * @exception JMSException
     */
    public static QueueConnection getQueueConnection(String qcfName, boolean started) 
            throws NamingException, JMSException {

        InitialContext ic = new InitialContext();
        QueueConnectionFactory qcf = (QueueConnectionFactory) 
                ic.lookup(qcfName);
        QueueConnection con = qcf.createQueueConnection();
        if (started) con.start(); 
        return con;
    }   

    /**
     * Returns a JMS TopicConnection object created after looking up the
     * given topic connection factory.
     * 
     * @param tcfName JNDI name of the topic connection factory that will
     * be used for creating the topic connection.
     * @param started If set to <code>true</code> then the new connection
     * is returned after caling <code>start()</code> on it.
     * @return TopicConnection
     * @exception NamingException
     * @exception JMSException
     */
    public static TopicConnection getTopicConnection(String tcfName, boolean started) 
            throws NamingException, JMSException {

        InitialContext ic = new InitialContext();
        TopicConnectionFactory tcf = (TopicConnectionFactory) 
                ic.lookup(tcfName);
        TopicConnection con = tcf.createTopicConnection();
        if (started) con.start();
        return con;
    }   

} 
