package org.bss.esb.util;


import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.methods.EntityEnclosingMethod;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.PutMethod;
import org.apache.commons.net.ftp.FTPClient;

import java.io.InputStream;
import java.net.URL;
import java.util.logging.Logger;

/**
 * Utility class that offers methods for performing the network operations
 * such as:
 * <pre>
 * 1. Http POST/GET/PUT operations
 * 2. Obtaining a client {@link FTPClient} for performing FTP operation.
 * </pre>
 * @version 1.0
 * @author Balwinder Sodhi
 * @see FTPClient
 * @see HttpClient
 */
public class NetUtil {
    
    /** Logger instance */
    private static Logger logger = Logger.getLogger(NetUtil.class.getName());

    /**
     * Does an HTTP POST of the resource available via the supplied
     * InputStream.
     * 
     * @param strURL URL for the request target
     * @param input InputStream for the resource to be POSTed
     * @param contentType
     * @param user User name for authenticating to target host
     * @param password Password for authenticating to target host
     * @return HTTP status code for the reply.
     * @throws Exception
     */     
    public static int doHttpPost(String strURL, InputStream input, 
            String contentType, String user, String password) 
                    throws Exception {
        
        URL url = new URL(strURL);
        // Get HTTP client
        HttpClient httpclient = new HttpClient();        
        
        // Pass our credentials to HttpClient, they will only be used for
        // authenticating to servers with a realm. To authenticate against
        // an arbitrary realm we use realm as null.
        httpclient.getState().setCredentials(null/*realm*/, url.getHost(),
                new UsernamePasswordCredentials(user, password));
        // Prepare HTTP post
        PostMethod post = new PostMethod(strURL);

        // Tell the POST method to automatically handle authentication. The
        // method will use any appropriate credentials to handle basic
        // authentication requests.  Setting this value to false will cause
        // any request for authentication to return with a status of 401.
        // It will then be up to the client to handle the authentication.
        post.setDoAuthentication(true);
        
        // Request content will be retrieved directly 
        // from the input stream
        post.setRequestBody(input);
        // Per default, the request content needs to be buffered
        // in order to determine its length.
        // Request body buffering can be avoided when
        // = content length is explicitly specified
        // = chunk-encoding is used
        post.setRequestContentLength(
                EntityEnclosingMethod.CONTENT_LENGTH_CHUNKED);
        // Specify content type and encoding
        // If content encoding is not explicitly specified
        // ISO-8859-1 is assumed
        if (contentType == null) {
                        
            contentType = "text/plain; charset=ISO-8859-1";
        }
        post.setRequestHeader("Content-type", contentType);
        // Execute request
        int result = httpclient.executeMethod(post);
        // Display status code
        logger.info("Response status code: " + result);
        // Display response
        logger.info("Response body: "+post.getResponseBodyAsString());
        // Release current connection to the connection pool once you are done
        post.releaseConnection();
        return result;        
    } 

    /**
     * Does an HTTP PUT of the resource available via the supplied
     * InputStream.
     * 
     * @param strURL URL for the request target
     * @param input InputStream for the resource to be PUT
     * @param contentType
     * @param user User name for authenticating to target host
     * @param password Password for authenticating to target host
     * @return HTTP status code for the reply.
     * @throws Exception
     */     
    public static int doHttpPut(String strURL, InputStream input, 
            String contentType, String user, String password) 
                    throws Exception {
        
        URL url = new URL(strURL);
        // Get HTTP client
        HttpClient httpclient = new HttpClient();        
        
        // Pass our credentials to HttpClient, they will only be used for
        // authenticating to servers with a realm. To authenticate against
        // an arbitrary realm we use realm as null.
        httpclient.getState().setCredentials(null/*realm*/, url.getHost(),
                new UsernamePasswordCredentials(user, password));
        // Prepare HTTP put
        PutMethod put = new PutMethod(strURL);

        // Tell the PUT method to automatically handle authentication. The
        // method will use any appropriate credentials to handle basic
        // authentication requests.  Setting this value to false will cause
        // any request for authentication to return with a status of 401.
        // It will then be up to the client to handle the authentication.
        put.setDoAuthentication(true);
        
        // Request content will be retrieved directly 
        // from the input stream
        put.setRequestBody(input);
        // Per default, the request content needs to be buffered
        // in order to determine its length.
        // Request body buffering can be avoided when
        // = content length is explicitly specified
        // = chunk-encoding is used
        put.setRequestContentLength(
                EntityEnclosingMethod.CONTENT_LENGTH_CHUNKED);
        // Specify content type and encoding
        // If content encoding is not explicitly specified
        // ISO-8859-1 is assumed
        if (contentType == null) {
                        
            contentType = "text/plain; charset=ISO-8859-1";
        }
        put.setRequestHeader("Content-type", contentType);
        // Execute request
        int result = httpclient.executeMethod(put);
        // Display status code
        logger.info("Response status code: " + result);
        // Display response
        logger.info("Response body: "+put.getResponseBodyAsString());
        // Release current connection to the connection pool once you are done
        put.releaseConnection();
        return result;        
    } 
    
    /**
     * Does an HTTP GET from the supplied URL. The data read in is
     * written into the OutputStream that we pass in.
     * 
     * @param strURL URL for the request target
     * @param user User name for authenticating to target host
     * @param password Password for authenticating to target host
     * @return byte[] The response body.
     * @throws Exception
     */     
    public static byte[] doHttpGet(String strURL, String user,
            String password) throws Exception {
        
        URL url = new URL(strURL);
        // Get HTTP client
        HttpClient httpclient = new HttpClient();        
        
        // Pass our credentials to HttpClient, they will only be used for
        // authenticating to servers with a realm. To authenticate against
        // an arbitrary realm we use realm as null.
        httpclient.getState().setCredentials(null/*realm*/, url.getHost(),
                new UsernamePasswordCredentials(user, password));
        // Prepare HTTP get
        GetMethod get = new GetMethod(strURL);

        // Tell the GET method to automatically handle authentication. The
        // method will use any appropriate credentials to handle basic
        // authentication requests.  Setting this value to false will cause
        // any request for authentication to return with a status of 401.
        // It will then be up to the client to handle the authentication.
        get.setDoAuthentication(true);
        
        // Execute request
        int result = httpclient.executeMethod(get);
        // Display status code
        logger.info("Response status code: " + result);
        
        byte[] response = get.getResponseBody();
        // Release current connection to the connection pool once you are done
        get.releaseConnection();
        return response;        
    } 

    /**
     * Gets the initialized {@link FTPClient} instance.
     * It connects to the default FTP port.
     * 
     * @param host - FTP host
     * @param port FTP port#
     * @param user - FTP user
     * @param passwd - FTP password
     * @return FTPClient - The initialized instance of FTPClient
     * @exception Exception
     */
    public static FTPClient getFtpClient(String host, int port, String user, 
            String passwd) throws Exception {

        logger.info("Called getFtpClient()");
        FTPClient ftpC = new FTPClient();
        ftpC.connect(host, port);
        ftpC.login(user, passwd);
        
        if (ftpC.setFileType(FTPClient.BINARY_FILE_TYPE)) {
            logger.severe("Could not set the transfer mode to BINARY. "+
                    ftpC.getReplyString());
        } else {
            logger.info("Transfer mode is set to BINARY");
        }
        logger.info("Opened the FTP session.");
        return ftpC;        
    }
    
    /**
     * Gets the initialized {@link FTPClient} instance.
     * It connects to the default FTP port (21).
     * 
     * @param host - FTP host
     * @param user - FTP user
     * @param passwd - FTP password
     * @return FTPClient - The initialized instance of FTPClient
     * @exception Exception
     */
    public static FTPClient getFtpClient(String host, String user, 
            String passwd) throws Exception {
                
        return getFtpClient(host, 21, user, passwd);
        
    }
    
    /**
     * Closes the {@link FTPClient} session and logs out from FTP server.
     * 
     * @param ftp - The FTPClient.
     */
    public static void closeFtpSession(FTPClient ftp) {
        
        try {

            ftp.logout();

        } catch (Exception exp) {
            //Do nothing;
        }
        try {

            ftp.disconnect();

        } catch (Exception exp) {
            //Do nothing;
        }                    
        logger.info("Closed the FTP session.");
        
    }

} 
